import oracle.sql.CLOB;
import java.sql.*;
import oracle.jdbc.driver.*;
import java.io.*;
import java.net.*;
import WriteCLOB;

public class InsertXMLDocument {

  public static void main( String[] arg ) throws Exception {

    // Connect to the Database
    Connection conn = Examples.getConnection();
    conn.setAutoCommit(false);

    String docStr      = "<test>document</test>";
    String filenameStr = "C:\\xmlbook\\src\\examples\\xml-basics-java\\test.xml";
    String urlStr      = "http://xml/xsql/demo/helloworld/helloworld.xsql";

    // Pass in a Reader for the String-based XML Document
    insertNewDocument(conn, "StringTest", new StringReader(docStr));

    // Pass in a Reader for the file-based XML Document
    insertNewDocument(conn, "FileTest", new FileReader(filenameStr));

    // Pass in a character input stream for an URL-based XML Document
    URL url = new URL(urlStr);
    insertNewDocument(conn, "URLTest", new InputStreamReader(url.openStream()));

    // Close the connection
    conn.close();
  }

  public static void insertNewDocument(Connection conn,
                                           String docname,
                                           Reader input) throws Exception {
    // Delete any existing doc (if any) with name 'docname' and
    // Insert a new row in the xml_documents table.
    // Perform the insert of the primary key and empty_clob()
    // as well as return the new CLOB locator in the same statement
    // using the RETURNING ... INTO ... clause.
    OracleCallableStatement stmt =
      (OracleCallableStatement) conn.prepareCall(
          "BEGIN " +
          "  DELETE FROM xml_documents WHERE docname = ?; "+
          "  INSERT INTO xml_documents( docname, xmldoc) " +
          "  VALUES( ?, empty_clob()) "+
          "  RETURNING xmldoc INTO ?;" +
          "END;");
    stmt.setString(1,docname); // Bind var in DELETE Statement
    stmt.setString(2,docname); // Bind var in VALUES()
    stmt.registerOutParameter(3,OracleTypes.CLOB); // RETURNING INTO

    stmt.execute();

    // Retrieve the returned values of DocId and CLOB locator
    CLOB theXMLClob = stmt.getCLOB(3);

    stmt.close();

    // Write the input to the CLOB
    WriteCLOB.fromReader( input, theXMLClob );

    // Commit the changes and close the connection.
    conn.commit();

    System.out.println("Inserted Docname '"+docname+"'.");
  }
}
