drop table forum_user;
drop table forum_topic_posting;
drop table forum_topic;
drop table forum;
drop sequence forumid;
create sequence forumid;
create table forum_user(
  userid     varchar2(10),
  email      varchar2(80),
  constraint forum_user_pk primary key (userid)
);
create table forum(
  id         number,
  created    date,
  name       varchar2(80),
  constraint forum_pk primary key (id)
);
create or replace trigger forum_id_trig
before insert on forum for each row
begin
  select forumid.nextval,sysdate
    into :new.id,:new.created
    from dual;
end;
.
/
create table forum_topic(
  id         number,
  forumid    number,
  created    date,
  title      varchar2(80) NOT NULL,
  userid     varchar2(80),
  postings   number,
  lastpost   date,
  constraint forum_topic_pk primary key (id),
  constraint topic_in_forum foreign key (forumid) references forum
);
create or replace trigger forum_topicid_trig
before insert on forum_topic for each row
begin
  select forumid.nextval,sysdate,0
    into :new.id,:new.created,:new.postings
    from dual;
end;
.
/
create table forum_topic_posting(
  id         number,
  topicid    number,
  userid     varchar2(30) NOT NULL,
  posted     date,
  posting    varchar2(4000) NOT NULL,
  constraint forum_topic_posting_pk primary key (id),
  constraint posting_for_topic      foreign key (topicid) references forum_topic
);
create or replace trigger forum_topic_postingid_trig
before insert on forum_topic_posting for each row
declare
  topicid      NUMBER;
  topicpostdate DATE := SYSDATE;
begin
  select forumid.nextval,topicpostdate
    into :new.id,:new.posted
    from dual;
  update forum_topic
     set postings = postings + 1,
         lastpost = topicpostdate
   where id = :new.topicid;  
end;
.
/
create or replace view forum_new_topic as
select t.forumid, t.title, t.userid, p.posting
  from forum_topic t,
       forum_topic_posting p
where t.id = p.topicid;
create or replace trigger forum_new_topic_ins
instead of insert on forum_new_topic for each row
declare
  newTopicId NUMBER;
begin
  insert into forum_topic(forumid,userid,title)
    values (:new.forumid,:new.userid, :new.title)
  returning id into newTopicId;
  insert into forum_topic_posting(topicid,userid,posting)
   values (newTopicId,:new.userid,:new.posting);
end;
.
/
insert into forum(name) values(
/* name      */ 'XML'
);
insert into forum(name) values(
/* name      */ 'Java'
);
insert into forum(name) values(
/* name      */ 'Business Components for Java'
);
insert into forum(name) values(
/* name      */ 'Oracle8i'
);
var id number
insert into forum_new_topic(forumid,title,userid,posting) values(
/* forumid   */ 1,
/* title     */ 'Welcome',
/* submitter */ 'smuench',
/* posting   */ 'Welcome to the XML Forum'
);
insert into forum_new_topic(forumid,title,userid,posting) values(
/* forumid   */ 2,
/* title     */ 'Welcome',
/* submitter */ 'smuench',
/* posting   */ 'Welcome to the Java Forum'
);
insert into forum_new_topic(forumid,title,userid,posting) values(
/* forumid   */ 3,
/* title     */ 'Welcome',
/* submitter */ 'smuench',
/* posting   */ 'Welcome to the Business Components for Java Forum'
);
insert into forum_new_topic(forumid,title,userid,posting) values(
/* forumid   */ 4,
/* title     */ 'Welcome',
/* submitter */ 'smuench',
/* posting   */ 'Welcome to the Oracle8i Forum'
);
commit;
CREATE INDEX forum_topic_posting_idx ON forum_topic_posting(posting)
INDEXTYPE IS ctxsys.context
PARAMETERS ('section group ctxsys.auto_section_group');
CREATE INDEX forum_topic_title_idx ON forum_topic(UPPER(title));

